<?xml version="1.0"?>
<xsl:stylesheet version="2.0" 
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:xs="http://www.w3.org/2001/XMLSchema">
	
<xsl:output method="html" version="4.0" encoding="UTF-8" indent="yes"/>

<xsl:param name="duration" select="P14D"/>
<xsl:param name="showResource"/>
<xsl:param name="withProjectImage"/>
<xsl:param name="withProjectStamp"/>
<xsl:param name="groupMode"/>
<xsl:param name="dateFormatter" select="'[M01]/[D01]/[Y]'"/>
<xsl:param name="durationFormatter" select="'#.00'"/>


<xsl:template name="formatDuration">
 	<xsl:param name="duration"/>

	<xsl:value-of select="format-number($duration/@amount,$durationFormatter)"/>
	<xsl:text> </xsl:text>
	<xsl:if test="$duration/@amount=1">
		<xsl:choose>
			<xsl:when test="$duration/@unit = 'minutes'">minute</xsl:when>
			<xsl:when test="$duration/@unit = 'hours'">hour</xsl:when>
			<xsl:when test="$duration/@unit = 'days'">day</xsl:when>
			<xsl:when test="$duration/@unit = 'weeks'">week</xsl:when>
			<xsl:when test="$duration/@unit = 'months'">month</xsl:when>
			<xsl:when test="$duration/@unit = 'quarters'">quarter</xsl:when>
			<xsl:when test="$duration/@unit = 'years'">year</xsl:when>
		</xsl:choose>
	</xsl:if>
	<xsl:if test="$duration/@amount!=1">
		<xsl:choose>
			<xsl:when test="$duration/@unit = 'minutes'">minutes</xsl:when>
			<xsl:when test="$duration/@unit = 'hours'">hours</xsl:when>
			<xsl:when test="$duration/@unit = 'days'">days</xsl:when>
			<xsl:when test="$duration/@unit = 'weeks'">weeks</xsl:when>
			<xsl:when test="$duration/@unit = 'months'">months</xsl:when>
			<xsl:when test="$duration/@unit = 'quarters'">quarters</xsl:when>
			<xsl:when test="$duration/@unit = 'years'">years</xsl:when>
		</xsl:choose>
	</xsl:if>
</xsl:template>

<xsl:template match="Project">
	<html>
	<head>
	<meta http-equiv="content-type" content="text/html; charset=utf-8" />
	<link rel="stylesheet" type="text/css" href="rsc/styles.css" />
	</head>
	<body>
	
	<!-- Standard Report Header for Merlin 2 Reports: START -->
	<xsl:if test="$withProjectStamp = 1">
		<div id="projectstamp">
		  <table class="report">
		  <tr>
			<th colspan="2">Project stamp</th></tr>
			<tr class="r1">
				<td class="title">Project name</td>
				<td class="projectname"><xsl:value-of select="title"/></td>
			</tr>
			<tr class="r0">
				<td class="title">Planned: Start-End</td>
				<td>
					<xsl:value-of select="format-dateTime(plannedStartDate, $dateFormatter)"/> - 
					<xsl:value-of select="format-dateTime(plannedEndDate, $dateFormatter)"/> 
					<br />(<xsl:value-of select="days-from-duration(xs:dayTimeDuration(plannedWork/Duration/@xmlDuration))"/> days / 
						   <xsl:value-of select="hours-from-duration(xs:dayTimeDuration(plannedWork/Duration/@xmlDuration))"/> hours work)
				</td>
			</tr>
			<tr class="r1">
				<td class="title">Expected: Start-End</td>
				<td>
					<xsl:value-of select="format-dateTime(expectedStartDate, $dateFormatter)"/> - 
					<xsl:value-of select="format-dateTime(expectedEndDate, $dateFormatter)"/>
					<xsl:if test="actualWork/Duration/@xmlDuration">
					<br />(<xsl:value-of select="days-from-duration(xs:dayTimeDuration(actualWork/Duration/@xmlDuration))"/> days /
						   <xsl:value-of select="hours-from-duration(xs:dayTimeDuration(actualWork/Duration/@xmlDuration))"/> hours work done)
					</xsl:if>
				</td>
			</tr>
			<xsl:if test="description != ''">
				<tr class="r0">
					<td class="title">Objective</td>
					<td><xsl:value-of select="description"/></td>
				</tr>
			</xsl:if>	
		</table>
		</div>
	</xsl:if> 	
	
	<xsl:if test="$withProjectImage = 1">
		<div id="logo">
			<img src="rsc/logo.png" />
		</div>
	</xsl:if> 	
	<!-- Standard Report Header for Merlin 2 Reports: END -->
	
	<div id="break" />
	
	<xsl:variable name="now" select="xs:dateTime(current-date())"/>
	<xsl:variable name="maxDate" select="$now + xs:dayTimeDuration($duration)"/>

	<xsl:if test="$showResource = ''">
		<h1>All Activities within the next <xsl:value-of select="days-from-duration($duration)"/> days</h1>
	</xsl:if>
	
	<xsl:if test="$showResource !='' ">
		<h1>Due Activities within the next <xsl:value-of select="days-from-duration($duration)"/> days for <xsl:value-of select="$showResource"/></h1>
	</xsl:if>

    <div id="nextduedates">

	<table class="report">
	<tr>
	<th>#</th>
	<th>Title</th>
	<th><xsl:if test="$groupMode = 0">Group</xsl:if><xsl:if test="$groupMode = 1">Path</xsl:if></th>
	<th>Status</th>
	<th>MS</th>
	<th>Start</th>
	<th><u>Deadline</u></th>
	<th>Work</th>
	<th>Duration</th>
	<th>Resources</th>
	</tr>
		
	<xsl:if test="$showResource = '' ">
		<xsl:for-each select="(//Activity | //Project) [not(Activity) and xs:dateTime(expectedEndDate) &lt;= xs:dateTime($maxDate) and xs:dateTime(expectedEndDate) &gt;= $now and actualCompletion &lt; 1.0]">
		<xsl:sort select="expectedEndDate"/>
		<tr class="r{position() mod 2}">
			<td><xsl:value-of select="@flatOrder"/></td>
			<td width="18%"><xsl:value-of select="title"/></td>
			<td width="18%">			
				<xsl:if test="$groupMode = 1">
					<xsl:if test="../../../../../../title"><xsl:value-of select="../../../../../../title"/>/</xsl:if><xsl:if test="../../../../../title"><xsl:value-of select="../../../../../title"/>/</xsl:if><xsl:if test="../../../../title"><xsl:value-of select="../../../../title"/>/</xsl:if><xsl:if test="../../../title"><xsl:value-of select="../../../title"/>/</xsl:if><xsl:if test="../../title"><xsl:value-of select="../../title"/>/</xsl:if><xsl:value-of select="../title"/>
					</xsl:if>
				<xsl:if test="$groupMode = 0"> <xsl:value-of select="../title"/></xsl:if>
			</td>			
			<td>
				<xsl:if test="flagStatus = 'red'">
					<img src="rsc/red.png" />
				</xsl:if>
				<xsl:if test="flagStatus = 'yellow'">
					<img src="rsc/yellow.png" />
				</xsl:if>
				<xsl:if test="flagStatus = 'green'">
					<img src="rsc/green.png" />
				</xsl:if>
			</td>
			<td>
				<xsl:if test="isMilestone">◆</xsl:if>
			</td>
			<td>
				<xsl:value-of select="format-dateTime(expectedStartDate, $dateFormatter )"/>
			</td>
			<td>
				<xsl:value-of select="format-dateTime(expectedEndDate, $dateFormatter )"/>
			</td>
			<td>
      			<xsl:call-template name="formatDuration">
      			  <xsl:with-param name="duration" select="expectedWork/Duration"/>
      			</xsl:call-template>
  			</td>
			<td>
     			<xsl:call-template name="formatDuration">
      			  <xsl:with-param name="duration" select="expectedDuration/Duration"/>
      			</xsl:call-template>
			</td>
			<td>
				<xsl:for-each select="./Assignment">
					<xsl:value-of select="title"/><br />
				</xsl:for-each>
			</td>
		</tr>
		</xsl:for-each>
	</xsl:if>
	
	<xsl:if test="$showResource != '' ">
		<xsl:for-each select="(//Activity | //Project) [./Assignment/title=$showResource and xs:dateTime(expectedEndDate) &lt;= xs:dateTime($maxDate) and xs:dateTime(expectedEndDate) &gt;= $now and actualCompletion &lt; 1.0]">
		<xsl:sort select="expectedEndDate"/>
		<tr class="r{position() mod 2}">
			<td><xsl:value-of select="@flatOrder"/></td>
			<td width="18%"><xsl:value-of select="title"/></td>
			<td width="18%">			
				<xsl:if test="$groupMode = 1">
					<xsl:if test="../../../../../../title"><xsl:value-of select="../../../../../../title"/>/</xsl:if><xsl:if test="../../../../../title"><xsl:value-of select="../../../../../title"/>/</xsl:if><xsl:if test="../../../../title"><xsl:value-of select="../../../../title"/>/</xsl:if><xsl:if test="../../../title"><xsl:value-of select="../../../title"/>/</xsl:if><xsl:if test="../../title"><xsl:value-of select="../../title"/>/</xsl:if><xsl:value-of select="../title"/>
					</xsl:if>
				<xsl:if test="$groupMode = 0"> <xsl:value-of select="../title"/></xsl:if>
			</td>			
			<td>
				<xsl:if test="flagStatus = 'red'">
					<img src="rsc/red.png" />
				</xsl:if>
				<xsl:if test="flagStatus = 'yellow'">
					<img src="rsc/yellow.png" />
				</xsl:if>
				<xsl:if test="flagStatus = 'green'">
					<img src="rsc/green.png" />
				</xsl:if>
			</td>
			<td>
				<xsl:if test="isMilestone">◆</xsl:if>
			</td>
			<td>
				<xsl:value-of select="format-dateTime(expectedStartDate, $dateFormatter )"/>
			</td>
			<td>
				<xsl:value-of select="format-dateTime(expectedEndDate, $dateFormatter )"/>
			</td>
			<td>
      			<xsl:call-template name="formatDuration">
      			  <xsl:with-param name="duration" select="expectedWork/Duration"/>
      			</xsl:call-template>
  			</td>
			<td>
     			<xsl:call-template name="formatDuration">
      			  <xsl:with-param name="duration" select="expectedDuration/Duration"/>
      			</xsl:call-template>
			</td>
			<td>
				<xsl:for-each select="./Assignment">
					<xsl:value-of select="title"/><br />
				</xsl:for-each>
			</td>
		</tr>
		</xsl:for-each>
	</xsl:if>
	
	</table>
	</div>
	
	<div id="copyright">
		Generated by Merlin — <a href="http://www.merlin2.net/" target="_blank">www.merlin2.net</a>
	</div>

  </body>
  </html>
 
</xsl:template>

</xsl:stylesheet>